# Detect the distribution in use
%global __despace head -n 1 | tr -d '[:space:]' | sed -e 's/[(].*[)]//g'
%global __lower4 cut -c 1-4 | tr '[:upper:]' '[:lower:]'
%global __distfile %([ -f /etc/SuSE-release ] && echo /etc/SuSE-release || echo /etc/redhat-release)
%global __distinit %(sed -e 's/ release .*//' -e 's/\\([A-Za-z]\\)[^ ]*/\\1/g' %{__distfile} | %{__despace} | %{__lower4})
%global __distvers %(sed -e 's/.* release \\([^. ]*\\).*/\\1/' %{__distfile} | %{__despace})
# Identify Alma, CentOS, CentOS Stream and Rocky Linux as rhel
%if "%{__distinit}" == "a" || "%{__distinit}" == "c" || "%{__distinit}" == "cl" || "%{__distinit}" == "cs" || "%{__distinit}" == "rl"
%global __distinit rhel
%endif
# Dist tag for Fedora is still "fc"
%if "%{__distinit}" == "f"
%global __distinit fc
%endif

# Set to 0 for regular curl package, 1 for libcurl compatibility package
%global compat 0

# Use libssh backend rather than libssh2 from Fedora 28 onwards
%if %([ 0%{?fedora} -gt 27 -o 0%{?rhel} -gt 7 ] && echo 1 || echo 0)
%global libssh libssh
%global libssh_minimum_version 0.7.5
%else
%global libssh libssh2
%global libssh_minimum_version 1.2
%endif

# Support websockets from Fedora 40 onwards
%if 0%{?fedora} > 39 || 0%{?rhel} > 9
%global websockets_support 1
%else
%global websockets_support 0
%endif

# Test suite wants python-impacket too (TODO get python3-impacket in EPEL-9 and use it)
%if %([ 0%{?fedora} -gt 28 -o 0%{?rhel} -gt 8 ] && echo 1 || echo 0)
%global python_impacket python3-impacket
%endif
%if %([ 0%{?fedora} -gt 0 -a 0%{?fedora} -le 28 ] && echo 1 || echo 0)
%global python_impacket %{nil}
%endif
%if %([ 0%{?rhel} -gt 0 -a 0%{?rhel} -le 9 ] && echo 1 || echo 0)
%global python_impacket %{nil}
%endif

Version: 8.7.0
Release: 1%{?dist}
%if %{compat}
Summary: Curl library for compatibility with old applications
Name: libcurl%(echo %{version} | tr -d .)
Obsoletes: compat-libcurl < %{version}-%{release}
Provides: compat-libcurl = %{version}-%{release}
%else
Summary: Utility for getting files from remote servers (FTP, HTTP, and others)
Name: curl
Provides: webclient
%endif
License: curl
Source0: https://curl.se/download/curl-%{version}.tar.xz

# Patch making libcurl multilib ready
Patch101: 0101-curl-7.85.0-multilib.patch

# Do not fail on warnings in the upstream test driver
Patch104: 0104-curl-8.1.0-tests-warnings.patch

# Fix FTBFS when building curl dynamically with no libcurl.so.4 in system
#Patch300: curl-7.64.1-zsh-cpl.patch

# Remove redundant compiler/linker flags from libcurl.pc
# Assumes %%{_libdir} = /usr/lib or /usr/lib64 and %%{_includedir} = /usr/include
Patch302: 0302-curl-7.74.0-pkgconfig.patch

URL: https://curl.se/
%if 0%{?fedora} > 28 || 0%{?rhel} > 7
BuildRequires: brotli-devel
%endif
BuildRequires: coreutils
BuildRequires: gcc
BuildRequires: krb5-devel
BuildRequires: libidn2-devel
BuildRequires: openldap-devel
BuildRequires: pkgconfig
BuildRequires: groff
BuildRequires: libnghttp2-devel >= 1.12.0
%if 0%{?fedora} > 33 || 0%{?rhel} > 8
BuildRequires: gnutls-devel >= 3.7.5
BuildRequires: libnghttp3-devel >= 1.0.0
BuildRequires: libngtcp2-devel >= 1.0.0
%endif
BuildRequires: libpsl-devel
BuildRequires: %{libssh}-devel >= %{libssh_minimum_version}
BuildRequires: make
BuildRequires: perl-interpreter
BuildRequires: sed
BuildRequires: openssl-devel
BuildRequires: zlib-devel
# Needed to compress content of tool_hugehelp.c after changing curl.1 man page
BuildRequires: perl(IO::Compress::Gzip)
# Needed for generation of shell completions
BuildRequires: perl(Getopt::Long)
BuildRequires: perl(Pod::Usage)
BuildRequires: perl(strict)
BuildRequires: perl(warnings)
# Using an older version of libcurl could result in CURLE_UNKNOWN_OPTION
Requires: libcurl%{?_isa} >= %{version}-%{release}
Requires: %{_sysconfdir}/pki/tls/certs/ca-bundle.crt
# Test suite requirements
#BuildRequires: glibc-langpack-en
BuildRequires: gnutls-utils
BuildRequires: hostname
# nghttpx (an HTTP/2 proxy) is used by the upstream test-suite
BuildRequires: nghttp2
BuildRequires: openssh-clients
BuildRequires: openssh-server
BuildRequires: perl(B)
BuildRequires: perl(base)
BuildRequires: perl(constant)
BuildRequires: perl(Cwd)
BuildRequires: perl(Digest::MD5)
BuildRequires: perl(Digest::SHA)
BuildRequires: perl(Exporter)
BuildRequires: perl(File::Basename)
BuildRequires: perl(File::Copy)
BuildRequires: perl(File::Spec)
BuildRequires: perl(IPC::Open2)
BuildRequires: perl(List::Util)
BuildRequires: perl(Memoize)
BuildRequires: perl(MIME::Base64)
BuildRequires: perl(POSIX)
BuildRequires: perl(Storable)
BuildRequires: perl(Time::HiRes)
BuildRequires: perl(Time::Local)
BuildRequires: perl(vars)
# python used for http-pipe tests (190x)
BuildRequires: python3-devel %{python_impacket}

# stunnel is used by upstream tests but it does not seem to work reliably
# on s390x and occasionally breaks some tests (mainly 1561 and 1562)
%ifnarch s390x
BuildRequires: stunnel
%endif

# require at least the version of libnghttp2 that we were built against,
# to ensure that we have the necessary symbols available (#2144277)
%global libnghttp2_version %(pkg-config --modversion libnghttp2 2>/dev/null || echo 0)

# require at least the version of libpsl that we were built against,
# to ensure that we have the necessary symbols available (#1631804)
%global libpsl_version %(pkg-config --modversion libpsl 2>/dev/null || echo 0)

# require at least the version of libssh/libssh2 that we were built against,
# to ensure that we have the necessary symbols available (#525002, #642796)
%global libssh_version %(pkg-config --modversion %{libssh} 2>/dev/null || echo 0)

# require at least the version of openssl-libs that we were built against,
# to ensure that we have the necessary symbols available (#1462184, #1462211)
# (we need to translate 3.0.0-alpha16 -> 3.0.0-0.alpha16 and 3.0.0-beta1 -> 3.0.0-0.beta1 though)
%global openssl_version %({ pkg-config --modversion openssl 2>/dev/null || echo 0;} | sed 's|-|-0.|')

%if ! %{compat}
%description
curl is a command line tool for transferring data with URL syntax, supporting
FTP, FTPS, HTTP, HTTPS, SCP, SFTP, TFTP, TELNET, DICT, LDAP, LDAPS, FILE, IMAP,
SMTP, POP3 and RTSP.  curl supports SSL certificates, HTTP POST, HTTP PUT, FTP
uploading, HTTP form based upload, proxies, cookies, user+password
authentication (Basic, Digest, NTLM, Negotiate, kerberos...), file transfer
resume, proxy tunneling and a busload of other useful tricks.

%package -n libcurl
Summary: A library for getting files from web servers
# libpsl adds symbols that curl uses if available, so we need to enforce
# version dependency
Requires: libpsl%{?_isa} >= %{libpsl_version}
# libssh/libssh2 adds symbols that curl uses if available, so we need to enforce
# version dependency
Requires: %{libssh}%{?_isa} >= %{libssh_version}
# same issue with openssl
Requires: openssl-libs%{?_isa} >= 1:%{openssl_version}
Requires: libnghttp2%{?_isa} >= %{libnghttp2_version}
%if 0%{?fedora} > 33 || 0%{?rhel} > 8
Requires: gnutls%{?_isa} >= 3.7.5
Requires: libnghttp3%{?_isa} >= 1.0.0
Requires: libngtcp2%{?_isa} >= 1.0.0
%endif

%description -n libcurl
libcurl is a free and easy-to-use client-side URL transfer library, supporting
FTP, FTPS, HTTP, HTTPS, SCP, SFTP, TFTP, TELNET, DICT, LDAP, LDAPS, FILE, IMAP,
SMTP, POP3 and RTSP. libcurl supports SSL certificates, HTTP POST, HTTP PUT,
FTP uploading, HTTP form based upload, proxies, cookies, user+password
authentication (Basic, Digest, NTLM, Negotiate, Kerberos4), file transfer
resume, HTTP proxy tunneling and more.

%package -n libcurl-devel
Requires: libcurl%{?_isa} = %{version}-%{release}
Requires: openssl-devel
Requires: %{libssh}-devel
Summary: Files needed for building applications with libcurl
Provides: curl-devel = %{version}-%{release}
Provides: curl-devel%{?_isa} = %{version}-%{release}
Obsoletes: curl-devel < %{version}-%{release}

%description -n libcurl-devel
The libcurl-devel package includes header files and libraries necessary for
developing programs that use the libcurl library. It contains the API
documentation of the library, too.

%package -n curl-minimal
Summary: Conservatively configured build of curl for minimal installations
Provides: curl = %{version}-%{release}
Conflicts: curl
#Suggests: libcurl-minimal
# Using an older version of libcurl could result in CURLE_UNKNOWN_OPTION
Requires: libcurl%{?_isa} >= %{version}-%{release}
#RemovePathPostfixes: .minimal
# Needed for RemovePathPostfixes to work with shared libraries
%undefine __brp_ldconfig

%description -n curl-minimal
This is a replacement of the 'curl' package for minimal installations. It
comes with a limited set of features compared to the 'curl' package. On the
other hand, the package is smaller and requires fewer run-time dependencies to
be installed.

%package -n libcurl-minimal
Summary: Conservatively configured build of libcurl for minimal installations
Provides: libcurl = %{version}-%{release}
Provides: libcurl%{?_isa} = %{version}-%{release}
Conflicts: libcurl%{?_isa}
#RemovePathPostfixes: .minimal
Requires: openssl-libs%{?_isa} >= 1:%{openssl_version}
Requires: libnghttp2%{?_isa} >= %{libnghttp2_version}
%if 0%{?fedora} > 33 || 0%{?rhel} > 8
Requires: gnutls%{?_isa} >= 3.7.5
Requires: libnghttp3%{?_isa} >= 1.0.0
Requires: libngtcp2%{?_isa} >= 1.0.0
%endif

%description -n libcurl-minimal
This is a replacement of the 'libcurl' package for minimal installations. It
comes with a limited set of features compared to the 'libcurl' package. On the
other hand, the package is smaller and requires fewer run-time dependencies to
be installed.
%else
%description
This package provides an old version of cURL libcurl library, necessary
for some old applications that have not been rebuilt against an up to date
version of cURL.
%endif

%prep
%setup -q -n curl-%{version}

# Upstream patches
# (none)

# Fedora Patches
%patch -P 101 -p1
%patch -P 104 -p1

# Local Patches
%patch -P 302

%build
export CPPFLAGS="$(pkg-config --cflags openssl)"
mkdir build-{full,minimal}
%global _configure ../configure
export common_configure_opts="        \
    --cache-file=../config.cache      \
    --disable-static                  \
    --enable-hsts                     \
    --enable-ipv6                     \
    --enable-symbol-hiding            \
    --enable-threaded-resolver        \
    --without-zstd                    \
    --with-gssapi                     \
    --with-libidn2                    \
    --with-nghttp2                    \
%if 0%{?fedora} > 33 || 0%{?rhel} > 8
    --with-gnutls                     \
    --with-nghttp3                    \
    --with-ngtcp2                     \
%else
    --with-openssl                    \
%endif
    --with-ca-bundle=%{_sysconfdir}/pki/tls/certs/ca-bundle.crt"

# configure minimal build
(
    cd build-minimal
    %configure $common_configure_opts \
    --disable-dict                    \
    --disable-gopher                  \
    --disable-imap                    \
    --disable-ldap                    \
    --disable-ldaps                   \
    --disable-docs                    \
    --disable-manual                  \
    --disable-mqtt                    \
    --disable-ntlm                    \
    --disable-ntlm-wb                 \
    --disable-pop3                    \
    --disable-rtsp                    \
    --disable-smb                     \
    --disable-smtp                    \
    --disable-telnet                  \
    --disable-tftp                    \
    --disable-tls-srp                 \
    --disable-websockets              \
    --without-brotli                  \
    --without-libpsl                  \
    --without-%{libssh}
)

# configure full build
(
    cd build-full
    %configure $common_configure_opts \
    --enable-dict                     \
    --enable-gopher                   \
    --enable-imap                     \
    --enable-ldap                     \
    --enable-ldaps                    \
    --disable-docs                    \
    --enable-manual                   \
    --enable-mqtt                     \
    --enable-ntlm                     \
    --enable-ntlm-wb                  \
    --enable-pop3                     \
    --enable-rtsp                     \
    --enable-smb                      \
    --enable-smtp                     \
    --enable-telnet                   \
    --enable-tftp                     \
    --enable-tls-srp                  \
%if %{websockets_support}
    --enable-websockets               \
%endif
%if 0%{?fedora} > 28 || 0%{?rhel} > 7
    --with-brotli                     \
%else
    --without-brotli                  \
%endif
    --with-libpsl                     \
    --with-%{libssh}
)

# Remove bogus rpath
sed -i \
    -e 's|^hardcode_libdir_flag_spec=.*|hardcode_libdir_flag_spec=""|g' \
    -e 's|^runpath_var=LD_RUN_PATH|runpath_var=DIE_RPATH_DIE|g' \
    build-{full,minimal}/libtool

%{make_build} -C build-minimal
%{make_build} -C build-full

%install
# Install and rename the library that will be packaged as libcurl-minimal
%{make_install} -C build-minimal/lib
rm -f %{buildroot}%{_libdir}/libcurl.{la,so}
for i in %{buildroot}%{_libdir}/*; do
    mv -v $i $i.minimal
done

# Install and rename the executable that will be packaged as curl-minimal
%{make_install} -C build-minimal/src
mv -v %{buildroot}%{_bindir}/curl{,.minimal}

# Install the executable and library that will be packaged as curl and libcurl
%{make_install} -C build-full

# Install zsh completion for curl
# (we have to override LD_LIBRARY_PATH because we eliminated rpath)
LD_LIBRARY_PATH="%{buildroot}%{_libdir}:$LD_LIBRARY_PATH" \
    %{make_install} -C build-full/scripts

# --disable-static not always honoured
rm -f %{buildroot}%{_libdir}/libcurl.a
install -d %{buildroot}%{_datadir}/aclocal
install -m 644 -p docs/libcurl/libcurl.m4 %{buildroot}%{_datadir}/aclocal

%if %([ 0%{?fedora} -lt 28 -a 0%{?rhel} -lt 8 ] && echo 1 || echo 0)
%if ! %{compat}
%post -n libcurl -p /sbin/ldconfig
%postun -n libcurl -p /sbin/ldconfig
%post -n libcurl-minimal -p /sbin/ldconfig
%postun -n libcurl-minimal -p /sbin/ldconfig
%else
%post -p /sbin/ldconfig
%postun -p /sbin/ldconfig
%endif
%endif

%files
%license COPYING
%doc CHANGES README*
%doc docs/BUGS.md docs/DEPRECATE.md docs/FAQ docs/FEATURES.md
%doc docs/TODO docs/TheArtOfHttpScripting.md
%if ! %{compat}
%{_bindir}/curl
%{_mandir}/man1/curl.1*
%else
%exclude %{_bindir}/curl
%exclude %{_mandir}/man1/curl.1*
%{_libdir}/libcurl.so.*
%endif

%if ! %{compat}
%files -n libcurl
%license COPYING
%{_libdir}/libcurl.so.4
%{_libdir}/libcurl.so.4.[0-9].[0-9]

%files -n libcurl-devel
%doc docs/examples/*.c docs/examples/Makefile.example docs/INTERNALS.md
%doc docs/CHECKSRC.md docs/CONTRIBUTE.md docs/libcurl/ABI.md docs/CODE_STYLE.md
%doc docs/GOVERNANCE.md
%{_bindir}/curl-config
%{_includedir}/curl/
%{_libdir}/*.so
%{_libdir}/pkgconfig/libcurl.pc
%{_mandir}/man1/curl-config.1*
%{_datadir}/aclocal/libcurl.m4

%files -n curl-minimal
%{_bindir}/curl.minimal
%{_mandir}/man1/curl.1*

%files -n libcurl-minimal
%license COPYING
%{_libdir}/libcurl.so.4.minimal
%{_libdir}/libcurl.so.4.[0-9].[0-9].minimal
%else
%exclude %{_bindir}/curl-config
%exclude %{_includedir}/curl/
%exclude %{_libdir}/*.so
%exclude %{_libdir}/pkgconfig/libcurl.pc
%exclude %{_mandir}/man1/curl-config.1*
%exclude %{_datadir}/aclocal/libcurl.m4
%endif
%if 0%{?fedora} < 36 && 0%{?rhel} < 10
%exclude %{_libdir}/libcurl.la
%endif

%changelog
* Wed Mar 27 2024 Teddysun <i@teddysun.com> - 8.7.0-1
- Update version to 8.7.0

* Wed Jan 31 2024 Teddysun <i@teddysun.com> - 8.6.0-1
- Update version to 8.6.0

* Wed Dec 06 2023 Teddysun <i@teddysun.com> - 8.5.0-1
- Update version to 8.5.0

* Mon Oct 23 2023 Teddysun <i@teddysun.com> - 8.4.0-2
- HTTP3 enabled (ngtcp2 + nghttp3) if fedora > 33 or rhel > 8
- option --with-gnutls replaced --with-openssl if fedora > 33 or rhel > 8

* Wed Oct 11 2023 Teddysun <i@teddysun.com> - 8.4.0-1
- Update to 8.4.0
